// Where's the Party At?
// Todd Michael Bailey
// Rev 1.01 firmware, first production run PCBs
// Mon Apr  6 16:01:14 CDT 2009

//=============================
// HOLLER-WARE LICENSE:
// Todd Bailey wrote this.  Do whatever you want with this code, but holler at me if you like it, use it, got a nice ride / big ol' butt, or know how to do it better.
// xoxoxo
// bai1ey.tm@gmail.com
//
// Todd Bailey would like to take this opportunity to shout out to:
//
// Todd Squires, who continues to be wholly intolerant of my bad programming habits,
// Limor Fried, for staying on my ass about making kits,
// BMT Toys and everybody there for putting me through the embedded-systems wringer for 3+ years of programming challenges, and for putting up with my chronic lateness and body-odor,
// Jim Williams, Paul Horowitz, Winfield Hill, and all the other people who've forgotten more than I'll ever know about all things analog,
// and most importantly,
// You, the Customer.
//=============================

//=============================
// Atmel AVR Atmega164p MCU, 5v operation.
// 20MHz Crystal Oscillator.
// Originally built with:
// AVR-Binutils 2.19,
// AVR-GCC 4.3.2,
// AVR-libc 1.6.4
//==============================

/*
Description:
==============================================================================
Just rock out, you know?


Changes and Updates from Last Revision:
==============================================================================

Fri Dec  5 14:12:30 CST 2008
Added new serial send and receive routines to accomodate helper IC.
Added new LED routines.
Changed softclock so it doesn't generate interrupts.
Changed UART routines slightly.
Changed the way the analog control input is monitored.
Changed program to accomodate new LED handling.
Changed key handling to accomodate helper IC.
MAJOR change to ISR to accomodate new parallel latch structure.
MAJOR change by combining redundant ANALOG and MIDI ISRs into a single Audio ISR accessed from multiple vectors.
Changed the names of a lot of silly things so they make sense, hopefully.


NOTES and for real ToDo stuff (Things That Make Me Go Hmm):
==============================================================================

Tue Apr  7 13:38:22 CDT 2009

Integrate Midi/Loop sampling (ISR) functions -- FIX OVERDUB
Integrate Midi/Loop sampling state machines
Set up new button and UI system
Set up 2-sample bank memory system
Write effects functions
Implement midi out fifo
Port gay PWM-style intro

Wed Apr  8 11:50:17 CDT 2009
	Bank System -- should be able to trigger playback/record/overdub independently in the two separate banks.
	Separate clock sources for the two banks, too.

Sat Mar 14 13:48:33 CDT 2009

Remove MCU-MCU serial communication code (may want to keep a variant around for debug)
	@@@ Make the output fifo used for the Helper into a MIDI out FIFO.
Implement MIDI out
Change Databus arbitration to include LED latch; may need to change RAM code also to accomodate this.
Include (again) analog pot handling on the ADC.
Include switch handling functions.
Change LED functions to include new latch.
Implement software jitter generator
Fix all DDRs
@@@ There are a lot of redundant settings of OE and WE and the LATCH_DDR.
@@@ There are too many calls to start and stop reading pots.
@@@ The loop sampler state machine is incredibly ugly.  Perhaps Include the LED blinking and pot reading code in the "Start Recording" functions if you can.
		There has to be a way to break those down more elegantly.  The midi code is a lot prettier.


Old stuff:
MIDI OVERDUB was never specified in the old version.
Remember to implement saving the MIDI channel to eeprom.
@@@ Might want to make the DAC recenter to mid-scale when it isn't spitting data.  This may or may not be necessesary based on whether or not the overdub input is DC coupled (right now it is) and how we ultimately end up doing overdubbing.
A great way to figure out where all the definitions for all the constants are kept locally is to try and redefine them and see how the compiler complains.  Just try defining PORTB as YOURMOM and see what happens.


Ideas, Indulgent Dreaming:
==============================================================================
	--------->  Analog clock in is on an Input Capture pin which can edge trigger -- could we also use this to change pitches up by, say fifths, right???  Cool....  Read up on this module.
	Nope, not really.  If you used the clock input (T1 or T0) you could do this with the output compare module...  Alas they're on PORTB.  Hmmm.
	Could still overclock and then divide in software, maybe.  We'd probably vector to the ISR way too often then.
	Could we mess with pitch by changing the Input Capture Edge?  Could double or halve it no problem that way, but other integers would suck.  Could also do this without polling by changing between ICP and PC interrupts.
	Halving the clock rate might also help the VCO be more tunable, too...  Hmm.  Could shift down by a 5th too by triggering on rise, not fall, not rise, fall.  Or something....
 	...Just playing it straight for now, trigger on rising edge.

	--------->  Could do a cool granular synthesis thing by dividing a sample by 2^EffectSelector, and shuffling the parts in playback.  Could also shuffle the array of pointers on a user command -- this would probably be better b/c a pattern
	would repeat until you told it to pick something new.  Would take a 256-byte long RAM table to shuffle.  Bigger part here we come....

	--------->  Now that banked samples are possible, it'll be cool to record more than one sample and then combine them in weird ways on playback (ie, not add, but multiply or XOR or whatever).
	--------->  Never did implement real-time effects.  Ought to.  Wonder if the RAM can get used for something (other than delay) with real time effects?  Like an envelope mask or something.
				Or, like the "banked sample" idea where you record one thing and then you multiply in real time against it.  Or something.
	VARIABLE crust, adjustable with the knob and not just ultimate crust.
	--------->	Should implement a tap-tempo one loop repeat feature

Nuts / Volts:
==============================================================================
Fuse bits:

EByte:  Don't change.
HByte:  0xD9	-- turn off JTAG.
LByte:  CKSEL=0111  SUT=11  CKDIV=1  CKOUT=1  so  1111 0111 or 0xF7

The device will then start at 20MHz.
I use:
avrdude -p m164p -c stk500v2 -u -t
r lfuse (should be 62)
w lfuse 0 0xF7
r hfuse (99 is default)
w hfuse 0 0xD9

*/

#include	"includes.h"

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//Application Defines:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// PORT / PIN Masks:
#define	LATCH_PORT		PORTB		// This is the all-purpose data port we use to write data to the RAM address and DAC latches and read and write from RAM.
#define	LATCH_DDR		DDRB		// It's associated DDR.
#define	LATCH_INPUT		PINB		// And the associated input latch.

// Input and output bitmasks:
// PORTD:
#define	Om_LED_LA			(1<<PD5)

// PORTA:
#define	Om_RAM_WE			(1<<PA1)
#define	Om_RAM_OE			(1<<PA2)
#define	Om_RAM_L_ADR_LA		(1<<PA3)
#define	Om_RAM_H_ADR_LA		(1<<PA4)
#define	Om_DAC_LA			(1<<PA5)

// Switch Input Bitmasks:
// These are on multiple ports, and the bit positions we describe here are the places we look for switchmasks in the application.
// The actual switch handling code worries about the hardware specifics.

#define		Im_SWITCH_0			(1<<0)
#define		Im_SWITCH_1			(1<<1)
#define		Im_SWITCH_2			(1<<2)
#define		Im_SWITCH_3			(1<<3)
#define		Im_SWITCH_4			(1<<4)
#define		Im_SWITCH_5			(1<<5)

// RAM constants
#define	MAX_RAM_ADDRESS	((unsigned long)0x7FFFF)	//	512k of SRAM here.

// Flags:
#define	CLK_INTERNAL		false	// Flags we use to determine what to set our clock source to when setting up an audio interrupt.
#define	CLK_EXTERNAL		true

// Switch Definitions by Function
// In the most recent sampler, the switches have a function that they more or less stick with.  I've renamed them to make it easier to
// tell what's going on when reading the code.

#define		Im_REC			Im_SWITCH_0
#define		Im_ODUB			Im_SWITCH_1
#define		Im_PLAY_PAUSE	Im_SWITCH_2
#define		Im_SINGLE_PLAY	Im_SWITCH_3
#define		Im_BANK			Im_SWITCH_4
#define		Im_EFFECT		Im_SWITCH_5


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Lists:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// Audio ISR States -- these are the different things we can do in the audio related ISR.
//---------------------------------------------------------------------------------------

enum
	{
		AUDIO_ISR_SAWTOOTH=0,
		AUDIO_ISR_RECORD,
		AUDIO_ISR_PLAYBACK,
		AUDIO_ISR_OVERDUB,
		MAX_AUDIO_ISR_FUNCTIONS,
	};


// ADC Channels:
//-----------------------------------------------------------------------
enum
{
	A_INPUT=0,
	A_OVERDUB,
	A_POT,
	NUM_ADCs,
};


// LEDs.
//-----------------------------------------------------------------------
enum					// LED enum used for keeping track of our LED masks.
	{
		LED_0=0,
		LED_1,
		LED_2,
		LED_3,
		LED_4,
		LED_5,
		LED_6,
		LED_7,
		NUM_LEDS,
	};

// WTPA has gotten to the point where each LED pretty much corresponds to an indicator of something specific.
// To reflect this, the masks here arrange LEDs by FUNCTION, to make code easier to read.

#define	Om_LED_REC			(1<<LED_0)
#define	Om_LED_ODUB			(1<<LED_1)
#define	Om_LED_PLAY			(1<<LED_2)
#define	Om_LED_OUT_OF_MEM	(1<<LED_3)
#define	Om_LED_BANK			(1<<LED_4)
#define	Om_LED_FX2			(1<<LED_5)
#define	Om_LED_FX1			(1<<LED_6)
#define	Om_LED_FX0			(1<<LED_7)

static unsigned char
	ledOnOffMask,		// What leds are on and off now?
	ledBlinkMask;		// What leds are blinking right now?


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Application Globals:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

// RAM address, data, and DAC stuff.
//-----------------------------------------------------------------------
static volatile unsigned long
	ramAddress;
static unsigned long
	sampleEndAddress;			// Where does the current sample end?  (all of them start at 0 for now).
static unsigned long
	overdubStartAddress;		// At what address did we start overdubbing?
static volatile unsigned char
	dacByte;


// Keys and analog reading variables.
//-----------------------------------------------------------------------
static unsigned char
	keyState,
	newKeys,
	potValue;		// Current value of our analog control pot input.

static bool
	updatePot;		// OK to read the analog pot now?


// Application flags and housekeeping.
//-----------------------------------------------------------------------
STATE_FUNC				//  Creates a pointer called State to an instance of STATE_FUNC().
	*State;
static unsigned char
	subState;			//  Keeps track of the minor states (sub states) the device can be in.

static volatile unsigned char
	audioIsrFunction;		// What should we be doing when the analog clock interrupt happens?

static volatile bool
	clockMode,				// Internal or external clock being used for the ISR?
	outOfRam;				// Goes true in the ISR if we run out of RAM.

static unsigned char		// Variables for effects.
	currentSelectedEffect;
static volatile bool
	loopOnce,
	halfBitDepth,
	halfSpeed,
	backwardsPlayback;

static bool					// Bank select variables.
	bank0,
	bank0Empty,
	bank1Empty;


// I hate Prototypes:
//-----------------------------------------------------------------------
// None so far....





//----------------------------------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------------------------------
// Da Code:
//----------------------------------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------------------------------




//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Interrupt Service Routines:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// This is where all the audio business happens in this program.
// Variables messed with in the ISRs AND mainline code should be declared volatile.

ISR(TIMER1_CAPT_vect)
// BIG PIMPIN AUDIO INTERRUPT:
// The analog clock triggers this interrupt when it's enabled, as does the MIDI related interrupt (internal TIMER1 compare A).
// Old timing benchmark-->
// Thu Apr 24 12:42:26 CDT 2008  NOTE: Watching the SCK line on the ramDac latches, it looks like this ISR takes about 22-24uS to complete or so.
// Thu Dec  4 18:48:47 CST 2008
// Changed all this shit to take out the shift register stuff.
// Also made the separate MIDI and ANALOG ISRs vector to the same code.  They did the same things, anyway.

// Fri Jan  2 18:26:06 CST 2009
// Time spent in ISR, checked with scope and toggling pin:
// Playback:  	7.5uS
// Odub:		9.0uS
// Record:		5.8uS
// Effects running take a little time, too, but not much.

// Sun Mar 15 22:44:46 CDT 2009
// By combining the LATCH and RAM DATA busses on the latest hardware, we introduce the following gotchas:
// 		The OE and WE lines may go high at any time between audio interrupts, and the DDR may turn into an output.  This is because of the LED latch being updated.
//		Therefore, we can set and leave those ports that way and expect they will stay that way, but anytime we want the DDR to be an input (reading from RAM) we must explicitly set it that way in the ISR.
//		The OE and WE lines can (and should, probably) idle high, so this isn't a big deal.
//		The value on the port itself may change also, so we should not rely on it being the same from ISR to ISR, which we weren't anyway.
{
	static volatile unsigned char
		temp;
	static bool
		flipFlop;

	switch(audioIsrFunction)
	{
		case AUDIO_ISR_SAWTOOTH:
		// Puts out a noble sawtooth wave.  Doesn't talk to the RAM at all.
		LATCH_DDR=0xFF;			// Data bus to output -- we never need to read the RAM in this version of the ISR.
		LATCH_PORT=dacByte;
		PORTA|=(Om_DAC_LA);		// Strobe dac latch enable high -- this puts the dacByte on the output...
		PORTA&=~(Om_DAC_LA);	// ...And keeps it there.
		dacByte++;				// Sawtooth away (should see a rising ramp if we haven't boned the hardware somehow).
		break;

		case AUDIO_ISR_RECORD:
		// Samples the current ADC channel and loads it into RAM.
		// Tristate the RAM's DATA lines before we get here; we'll never need them to be OPs. (OE high)
		// Set the AVR's DDRs on the DATA port to outputs.  That won't change here either.
		// NOTE:  We are explicitly NOT checking to see if the last ADconversion is done when starting a new one so that weird stuff will happen if we overclock this channel.
		// NOTE:  This doesn't put out audio because dacByte doesn't change, so there's no AC signal.
		LATCH_DDR=0xFF;			// Data bus to output -- we never need to read the RAM in this version of the ISR.

		temp=ADCH;							// Store 8 high bits of the last conversion.
		ADCSRA |= (1<<ADSC);  				// Start the next conversion.

		LATCH_PORT=(ramAddress&0xFF);		// Put the LSB of the address on the latch.
		PORTA|=(Om_RAM_L_ADR_LA);			// Strobe it to the latch output...
		PORTA&=~(Om_RAM_L_ADR_LA);			// ...Keep it there.

		LATCH_PORT=((ramAddress>>8)&0xFF);	// Put the middle byte of the address on the latch.
		PORTA|=(Om_RAM_H_ADR_LA);			// Strobe it to the latch output...
		PORTA&=~(Om_RAM_H_ADR_LA);			// ...Keep it there.

//		portShadow=PINC;					// Get the state of Port C (where our three MSbs are for the RAM address are).
//		portShadow&=0xF8;					// Clear the three ram address pins we care about from the shadow register.
//		portShadow|=((ramAddress>>16)&0x07);// Put the correct address on the 3 LSbs of the shadow register.
//		PORTC=portShadow;					// Put the shadow back on the port.  This completes setting the RAM address.

		PORTC=(0xF8|((ramAddress>>16)&0x07));	// Only switches on PORTC now.

		LATCH_PORT=temp;					// Put the data to write on the RAM's input port

	MACRO_DoTenNops						// @@@

		PORTA&=~(Om_RAM_WE);				// Strobe Write Enable low.  This latches the data in.
	MACRO_DoTenNops						// @@@
		PORTA|=(Om_RAM_WE);					// Disbale writes.

		ramAddress++;						// Next address please.
		if(ramAddress>MAX_RAM_ADDRESS)		// Out of RAM?
		{
			ramAddress=MAX_RAM_ADDRESS;		// Don't go any farther.
			outOfRam=true;					// Signal mainline code that we're out of memory.
		}
		break;

		case AUDIO_ISR_PLAYBACK:
		// Goes through RAM and spits out bytes, looping from 0 to sampleEndAddress.
		// Since we cannot count on the OE staying low or the AVR's DDR remaining an input, we will explicitly set all the registers we need to every time through this ISR.

		// @@@ OLD AND WRONG: We should begin this with the MCU's DATA port tri-stated, the RAM's output enabled (OE low), and writes disabled.  To playback we just put the address on the lines and read the data port.

		if((halfSpeed==false)||(halfSpeed&&flipFlop))		// Play the sample back every ISR if we aren't at half speed, OR play the sample back every other time if we are.
		{
			LATCH_PORT=(ramAddress&0xFF);		// Put the LSB of the address on the latch.
			PORTA|=(Om_RAM_L_ADR_LA);			// Strobe it to the latch output...
			PORTA&=~(Om_RAM_L_ADR_LA);			// ...Keep it there.

			LATCH_PORT=((ramAddress>>8)&0xFF);	// Put the middle byte of the address on the latch.
			PORTA|=(Om_RAM_H_ADR_LA);			// Strobe it to the latch output...
			PORTA&=~(Om_RAM_H_ADR_LA);			// ...Keep it there.

//			portShadow=PINC;					// Get the state of Port C (where our three MSbs are for the RAM address are).
//			portShadow&=0xF8;					// Clear the three ram address pins we care about from the shadow register.
//			portShadow|=((ramAddress>>16)&0x07);// Put the correct address on the 3 LSbs of the shadow register.
//			PORTC=portShadow;					// Put the shadow back on the port.  This completes setting the RAM address.

			PORTC=(0xF8|((ramAddress>>16)&0x07));	// Only switches on PORTC now.


			LATCH_DDR=0x00;						// Turn the data bus around (AVR's data port to inputs)
			PORTA&=~(Om_RAM_OE);				// RAM's IO pins to outputs.

//asm volatile("nop"::);
//asm volatile("nop"::);
// @@@ This is the magic spot for turn around time. All the other delays are unnecessary but we need at least this much delay (two nops) here for bus turnaround.  No delay gives no signal, and one nop is crusty.  Try to elegant this up later.
		MACRO_DoTenNops						// @@@
			dacByte=LATCH_INPUT;				// Get the byte from this address in RAM.
		MACRO_DoTenNops						// @@@
			PORTA|=(Om_RAM_OE);					// Tristate the RAM.
			LATCH_DDR=0xFF;						// Turn the data bus around (AVR's data port to outputs)

			if(halfBitDepth)
			{
				dacByte&=0x80;		// Mask off top bit.
			}

			LATCH_PORT=dacByte;		// Put the dacByte on the output latch's input.
			PORTA|=(Om_DAC_LA);		// Strobe dac latch enable high -- this puts the dacByte on the 373's output...
			PORTA&=~(Om_DAC_LA);	// ...And keeps it there.

			if(backwardsPlayback)
			{
				if((loopOnce==false)||(ramAddress!=0))
				{
					if(ramAddress==0)					// Roll around to the end of the sample.
					{
						ramAddress=sampleEndAddress;	// Back to the, um, future.
					}
					else
					{
						ramAddress--;					// Previous address.
					}
				}
			}
			else
			{
				if((loopOnce==false)||(ramAddress!=sampleEndAddress))
				{
					ramAddress++;						// Next address.
					if(ramAddress>sampleEndAddress)		// If the sample was the entire buffer, we'll roll around anyway.
					{
						ramAddress=0;				// Back to the beginning.
					}
				}
			}
		}

		if(flipFlop)			// Toggle our half-speed flip flop.
		{
			flipFlop=false;
		}
		else
		{
			flipFlop=true;
		}
		break;

		case AUDIO_ISR_OVERDUB:
		// OG Method (conserves RAM, destructive overdub):
		// This gets the data from RAM and then puts it on the DAC like the playback routine, then overwrites the RAM location with the results of the ADC (the ADC here reads a sum of the DAC and user input).
		// When working this way, the maximum overdub length is the length of the original sample.  This rolling buffer starts wherever the OVERDUB is punched in and stops whenever the user says, or
		// when we've rolled around and gotten back to the OVERDUB start address.
		// I guess there's no reason not to just keep recording until the user says to stop.  Probably should make this an option.

		// New School "I'm Stanky With RAM" Method (uses more RAM, slower, allows non-destructive record, allows layering a longer pattern over a shorter repeated pattern):
		// Here we get dacByte from the first sample's address RAM and PLAY it, but we RECORD to a new location in RAM.  We can keep looping the first sample as long as we want without changing it.
		// (If we punch out before the first sample is done we'll have to record the remainder of the first sample into the new address...  The only way to really do this, I think, would be to keep
		// sampling but mute the user's input.  Reassign hardware ahoy.

		// NOTE:  FOR THIS TO NOT SUCK, we need to be able to assign all the playback effects while overdubbing.  We didn't do this in the old version.

		// Sun Mar 15 23:04:38 CDT 2009
		// The ODUB must also observe the same bus arbitration as the playback -- see above.

		if((halfSpeed==false)||(halfSpeed&&flipFlop))		// Play the sample back every ISR if we aren't at half speed, OR play the sample back every other time if we are.
		{
			// Get ADC data and start new conversion:
			temp=ADCH;							// Store 8 high bits of the last conversion.
			ADCSRA |= (1<<ADSC);  				// Start the next conversion.


			LATCH_PORT=(ramAddress&0xFF);		// Put the LSB of the address on the latch.
			PORTA|=(Om_RAM_L_ADR_LA);			// Strobe it to the latch output...
			PORTA&=~(Om_RAM_L_ADR_LA);			// ...Keep it there.

			LATCH_PORT=((ramAddress>>8)&0xFF);	// Put the middle byte of the address on the latch.
			PORTA|=(Om_RAM_H_ADR_LA);			// Strobe it to the latch output...
			PORTA&=~(Om_RAM_H_ADR_LA);			// ...Keep it there.

//			portShadow=PINC;					// Get the state of Port C (where our three MSbs are for the RAM address are).
//			portShadow&=0xF8;					// Clear the three ram address pins we care about from the shadow register.
//			portShadow|=((ramAddress>>16)&0x07);// Put the correct address on the 3 LSbs of the shadow register.
//			PORTC=portShadow;					// Put the shadow back on the port.  This completes setting the RAM address.

			PORTC=(0xF8|((ramAddress>>16)&0x07));	// Only switches on PORTC now.


			LATCH_DDR=0x00;						// Turn the data bus around (AVR's data port to inputs)
			PORTA&=~(Om_RAM_OE);				// RAM's IO pins to outputs.
		MACRO_DoTenNops						// @@@
			dacByte=LATCH_INPUT;				// Get the byte from this address in RAM.
		MACRO_DoTenNops						// @@@
			PORTA|=(Om_RAM_OE);					// Tristate the RAM.
			LATCH_DDR=0xFF;						// Turn the data bus around (AVR's data port to outputs)

			if(halfBitDepth)
			{
				dacByte&=0x80;		// Mask off top bit.
			}

			LATCH_PORT=dacByte;		// Put the dacByte on the output latch's input.
			PORTA|=(Om_DAC_LA);		// Strobe dac latch enable high -- this puts the dacByte on the 373's output...
			PORTA&=~(Om_DAC_LA);	// ...And keeps it there.

			LATCH_PORT=temp;		// Now replace the data at this RAM location with the data from the ADC.
		MACRO_DoTenNops						// @@@
			PORTA&=~(Om_RAM_WE);	// Strobe Write Enable low.  This latches the data in.
		MACRO_DoTenNops						// @@@
			PORTA|=(Om_RAM_WE);		// Disbale writes.

						
			if(backwardsPlayback)
			{
				if((loopOnce==false)||(ramAddress!=0))
				{
					if(ramAddress==0)					// Roll around to the end of the sample.
					{
						ramAddress=sampleEndAddress;	// Back to the, um, future.
					}
					else
					{
						ramAddress--;					// Previous address.
					}
				}
			}
			else
			{
				if((loopOnce==false)||(ramAddress!=sampleEndAddress))
				{
					ramAddress++;						// Next address.
					if(ramAddress>sampleEndAddress)		// If the sample was the entire buffer, we'll roll around anyway.
					{
						ramAddress=0;				// Back to the beginning.
					}
				}
			}
		}

		if(flipFlop)			// Toggle our half-speed flip flop.
		{
			flipFlop=false;
		}
		else
		{
			flipFlop=true;
		}
		break;

		default:	// Should never happen.
		break;
	}
}

ISR(TIMER1_COMPA_vect, ISR_ALIASOF(TIMER1_CAPT_vect));	// @@@	// This just tells the timer1 compare match interrupt to point at the code in the Timer1 Input Capture interrupt.  We use different interrupt vectors to get to the audio handling code depending on sampler's current mode.

ISR(__vector_default)
{
    //  This means a bug happened.  Some interrupt that shouldn't have generated an interrupt went here, the default interrupt vector.
	//	printf("Buggy Interrupt Generated!  Flags = ");
	//  printf("*****put interrupt register values here****");
}


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// State Machine Functions.
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

static void SetState(STATE_FUNC *newState)		// Sets the device to a new state, assumes it should begin at the first minor sub-state.
{
	State=newState;
	subState=SS_0;
}

//-----------------------------------------------------------------------
// LED functions:
//-----------------------------------------------------------------------

static void SetLedsOnOff(unsigned char theMask)
// Sets the LEDs to be on or off according to the bitmask.  This mask is sent out to the LEDs in the main loop.
{
	ledOnOffMask=theMask;
}

static void BlinkLeds(unsigned int theMask, unsigned long blinkTime)
// Sets up the mask of leds to blink and their blink rate.
// NOTE:  All leds made to blink will blink synchronously at the rate last set.  This saves us from having 8 separate software clocks.
// NOTE:  When an LED is told to stop blinking it will revert to OFF, even if the LED was ON when we told it to blink.  I can live with this.
{
	unsigned char 
		i;
	
	for(i=0; i<NUM_LEDS; i++)		// Go through all the LEDs...
	{
		if(ledBlinkMask&(1<<i))		// ...and if a LED was supposed to be blinking...
		{
			ledOnOffMask&=~(1<<i);		// ...turn it off.  This makes sure any blinking LEDs don't stop their blinks in the 'on' state.	
		}	
	}

	ledBlinkMask=theMask;				// Now assign new leds to blink.
	SetTimer(TIMER_BLINK,blinkTime);	// And the time that they will all blink to.
}

static void StopBlinking(void)
// Stops all blinking LEDs.
{
	BlinkLeds(0,0);		// Durrrr.....
}

static void KillLeds(void)
// Turns off all LEDs immediately.
{
	ledOnOffMask=0;
	BlinkLeds(0,0);		// Durrrr.....
}

static void WriteLedLatch(unsigned char theMask)
// Take the current on/off LED mask and put it onto the LED output latch.
// This is the only mainline (non IRQ) code that messes with the databus.
{
	unsigned char
		sreg;

	sreg=SREG;	// Store global interrupt state
	cli();		// Clear global interrupts (so we don't get an audio interrupt while messing with OE and WE)
	
	PORTA|=(Om_RAM_OE|Om_RAM_WE);	// Disable reading and writing to RAM.
	LATCH_PORT=theMask;				// Put passed data onto bus.
	LATCH_DDR=0xFF;					// Make sure the bus is an output.
	PORTD|=(Om_LED_LA);				// Strobe it to the latch output...
	PORTD&=~(Om_LED_LA);			// ...Keep it there.
	
	SREG=sreg;						// Restore interrupts.
}

static void HandleLeds(void)
// Runs in the main loop updating the state of the LEDs.  Only messes with the databus when there's something to change.
{
	unsigned char 
		i;
	static bool
		toggle;				// Flip flop for blinking.
	static unsigned char
		lastLedMask=0;		// Used to see if LEDs have been changed during a given loop.

	if(ledBlinkMask&&CheckTimer(TIMER_BLINK))		// Are we blinking and is it time to toggle?
	{
		for(i=0; i<NUM_LEDS; i++)	// Which LEDs are we blinking?
		{
			if(ledBlinkMask&(1<<i))
			{
				if(toggle)			// Flip flop the LEDs we're blinking every time the timer runs out.
				{
					ledOnOffMask|=(1<<i);	
				}			
				else
				{
					ledOnOffMask&=~(1<<i);
				}
			}
		}

		toggle=(!toggle);				// flip the sign of the led for next time.
		ResetTimer(TIMER_BLINK);		// And wait until next time.
		WriteLedLatch(ledOnOffMask);	// Send the LED value to the latch.	
	}
	else if(lastLedMask!=ledOnOffMask) // If we're not blinking, but our LEDs have been instructed to change...
	{
		WriteLedLatch(ledOnOffMask);	// ...send the LED value to the latch.	
		lastLedMask=ledOnOffMask;		// And mark it as sent.
	}
}

static void InitLeds(void)
{
	ledOnOffMask=0;
	ledBlinkMask=0;
	WriteLedLatch(0);	// ...send the LED value to the latch.	
}


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Switch functions:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

static void InitSwitches(void)
{
	MCUCR&=~(1<<PUD);	// Globally enable pullups (should be default)
	DDRD&=~(1<<PD7);	// PD7 to input
	DDRC&=~(0xF8);		// PC3-7 to input
	PORTD|=(1<<PD7);	// Pullup on.
	PORTC|=(0xF8);		// Pullups on.

	SetTimer(TIMER_DEBOUNCE,(SECOND/32));
}

static void HandleSwitches(void)
// Read input pins, debounce, make keypresses positive-true, and flag newly-appeared keys.
{
	static unsigned char
		lastKeyState;


	if(CheckTimer(TIMER_DEBOUNCE))
	{
		keyState=(PIND&0x80)>>7;		// Switch 0 (PIND, bit 7)
		keyState|=(PINC&0xF8)>>2;		// Switches 1-5 (PINC bits 3-7)
		keyState=(~keyState)&0x3F;		// Flip bits to make low input level a 1, then mask off unused bits.
		SetTimer(TIMER_DEBOUNCE,(SECOND/32));
	}

	newKeys=((keyState^lastKeyState)&(keyState));		// Flag the keys which have been pressed since the last test.
	lastKeyState=keyState;								// And store this keystate as old news.
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// A/D Control Functions:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// WTPA uses 3 analog inputs (ADC6,7, and 0).  The first is used to sample the audio input.  The second is used to sample the summed output of the input and the DAC when we're overdubbing.
// The third is used to sample the user input pot (the bootleg shuttlewheel).
// The best resolution we can get from this hardware is 10 bits, +/- 2 lsbs.
// The max sampling rate we can pull at full resolution is 15kHz.  We always use the ADC single ended (so far) as the datasheet says that the ADC isn't guaranteed to
// work in differential mode with the PDIP package.  A conversion takes 13 ADC clocks normally, or 25 for the first conversion after the channel is selected.
// The datasheet is unclear how much resolution is lost above 15kHz.  Guess we'll find out!
// NOTE:  Since the RAM can only store 8 bits per sample, we're only using 8 bits of the conversion data.  Can't imagine those last two bits are going to be worth much at these sample rates anyhoo.

// WARNING:  It is the caller's responsibility to NOT point the ADC at the pot while sampling is taking place, and to make sure that polling is disabled.

static void SetAdcChannel(unsigned char theChannel)
{
	switch(theChannel)
	{
		case A_INPUT:
		ADMUX=(ADMUX&0xF0);		// Clear the mux bits that we care about.
		ADMUX|=0x06;			// ADC6 selected.
		break;

		case A_OVERDUB:
		ADMUX=(ADMUX&0xF0);		// Clear the mux bits that we care about.
		ADMUX|=0x07;			// ADC7
		break;

		case A_POT:
		ADMUX=(ADMUX&0xF0);		// Clear the mux bits that we care about, and leave them.  This selects ADC0.
		break;
/*
		default:				// You goofed.  Shouldn't happen.
		ADMUX=(ADMUX&0xF8);		// Clear the mux bits that we care about.
		ADMUX|=0x06;			// ADC6 selected.
		break;
*/
	}
}

static void StopReadingPot(void)
// Stop reading the pot from the ADC.
{
	updatePot=false;
}

static void StartReadingPot(void)
// Set up to read the pot on the ADC.
{
	SetAdcChannel(A_POT);		// Set the channel correctly.
	ADCSRA |= (1<<ADSC);  		// Start the next conversion.				
	updatePot=true;				// Raise flag for the update routine to poll.
}

static void HandleReadingPot(void)
// Sits in the background and periodically updates the analog control input.
// Right now we only do this when we're playing back and never when we're sampling, so it's OK to pre-empt the ADC for this.
// We're also not doing any oversampling...  Yet.
{
	if(updatePot)						// Is it OK to be messing with this right now?
	{
		if(!(ADCSRA&(1<<ADSC)))			// Last conversion done?
		{
			potValue=ADCH;				// Store it.
			ADCSRA |= (1<<ADSC);  		// Start the next conversion.				
		}
	}
}

/*
static void UnInitAdc(void)
{
	ADCSRA&=~(1<<ADEN);		// Disable ADC.
	PRR|=(1<<PRADC);		// Power down the ADC.
}
*/

static void InitAdc(void)
// Note, we don't set up the Adc to trigger on anything right away.
{
	PRR&=~(1<<PRADC);		// Turn the ADC power on (clear the bit to power on).
	ADMUX = 0x66; 			// 0110 0110.  AVCC is the reference (w/ ext cap), left justify the result, start with ADC6 as the channel.  The ADC always wants to see a cap at AREF if the internal references or VCC are used.
	DIDR0 = 0xC1;			// Disable the digital inputs for ADC6,ADC7, and ADC0.
	ADCSRA = 0x96;			// 1001 0110 (prescaler to 64 = 24kHz sample rate, (32 = 48kHz).  Enable the ADC, no autotrigger or interrupts. (For ex: at 8MHz sysclk and 1/64, ADC clock = 125kHz, normal conversion ~= 10kHz (13 samples per conversion).  According to the datasheet, to get max resolution from the converter, the ADC clock must be between 50 and 200kHz.)

	ADCSRA |= (1<<ADSC);  	// Start the first conversion to initialize the ADC.
	updatePot=false;		// Don't read the pot yet.

	// ADCSRB controls auto-triggering, which we aren't using right now.
}

//-----------------------------------------------------------------------
// RAM / DAC functions:
//-----------------------------------------------------------------------
// Will be totally different than the oldschool serial biz.....
// Should probably all be inlined in the ISR.
// Just so you remember, Parallel SRAM pretty much always works like this:

// The chip is always enabled (CS is always low).  The other two control pins are active low also -- they are Write Enable and Output Enable.
// When WE is low, the value on the DATA pins is latched into the address on the ADDRESS pins.  Asserting WE will also override OE and make the RAM's DATA Pins go high impedance.
// When WE is high, the byte stored at the ADDRESS on pins A0-A18 is latched out on the DATA pins, assuming OE is low.
// If OE is high, the DATA port will be high impedance no matter what.
// The ADDRESS pins can always be outputs as far as the MCU is concerned.

// A typical write might look like:
// 1.)  OE and WE are high.
// 2.)  The address is set and the data latches on the MCU are made into outputs. The correct values of Data and Address are set on the corresponding pins.
// 3.)  WE is brought low and the data is latched in.
// 4.)  WE is brought high.  The address and data lines can now be changed without messing up data on the RAM chip.

// A typical read might look like this:
// 1.)  OE and WE are high.
// 2.)  The address is set correctly and the DATA PORT on the MCU is made high impedance.
// 3.)  OE is brought low, and the data to be read shows up on the DATA lines.
// 4.)  The MCU reads the DATA lines.
// 5.)  OE can be brought high again or left low -- it only needs to change if we're going to write to the chip.

// On 373 Parallel Latches:
// These guys are pretty simple.  In this circuit, their OE is tied low (enabled).
// While LE (Latch Enable) is High, the Latch is transparent from input port to output port.
// When LE is brought low, the current state of the inputs will be latched, and the outputs will that hold value while LE remains low (there's a little delay but it isn't relevant for us).
// So, we will will probably just leave LE low most of the time on the latches and strobe it high when we want to update that particular latch.

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// General Sampler/ISR Functions:
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

static void StartRecording(unsigned int theRate)
// Set the memory pointer to the start of RAM, set up the clock source, set the interrupt to the recording handler, and enable interrupts.
// If we're using the internal clock, set the rate.
{
	ramAddress=0;						// Always start recording at the beginning of RAM.
	outOfRam=false;						// Plenty of ram left...
	SetAdcChannel(A_INPUT);				// Point ADC at the input.

	audioIsrFunction=AUDIO_ISR_RECORD;	// What should we be doing when we get into the ISR?
	PORTA|=(Om_RAM_OE);					// Tristate the RAM's IO pins.
	PORTA|=(Om_RAM_WE);					// Disbale writes.
	LATCH_DDR=0xFF;						// MCU's data port to outputs.

	if(clockMode==CLK_INTERNAL)			// The internally counted clock is usually associated with MIDI-controlled sampling. 
	{
		TCNT1=0;					// Initialize the counter to 0.
		OCR1A=theRate;				// Set the compare register to our presupposed record rate.
		TIFR1=0xFF;					// Clear the interrupt flags by writing ones.
		TIMSK1|=(1<<OCIE1A);		// Enable the compare match interrupt.
		TCCR1B=0x09;				// Start the timer in CTC mode, prescaler at 1/1
	}
	else
	{
		TCCR1B|=(1<<ICES1);		// Trigger on a rising edge.
		TIFR1=0xFF;				// Clear interrupts.
		TIMSK1|=(1<<ICIE1);		// Enable Input Capture Interrupt (yo, son, I thought I was the Icy One?)
	}
}

static void StartPlayback(unsigned int theRate)
// Point to the beginning of RAM, select the clock source, and get the interrupts going.
// Set the clock rate if we're using the internal clock.
{
	ramAddress=0;							// Start playback at the beginning of RAM.
	audioIsrFunction=AUDIO_ISR_PLAYBACK;	// What should we be doing when we get into the ISR?
	PORTA|=(Om_RAM_OE);						// Tristate the RAM's IO pins.
	PORTA|=(Om_RAM_WE);						// Disbale writes.
	LATCH_DDR=0xFF;							// MCU's data port to outputs.

	if(clockMode==CLK_INTERNAL)		// The internally counted clock is usually associated with MIDI-controlled sampling. 
	{
		TCNT1=0;					// Initialize the counter to 0.
		OCR1A=theRate;				// Set the compare register for the appropriate clock rate.
		TIFR1=0xFF;									// Clear the interrupt flags by writing ones.
		TIMSK1|=(1<<OCIE1A);						// Enable the compare match interrupt.
		TCCR1B=0x09;								// Start the timer in CTC mode, prescaler at 1/1
	}
	else
	{
		TCCR1B|=(1<<ICES1);		// Trigger on a rising edge.
		TIFR1=0xFF;				// Clear interrupts.
		TIMSK1|=(1<<ICIE1);		// Enable Input Capture Interrupt (yo, son, I thought I was the Icy One?)
	}
}

static void ContinuePlayback(unsigned int theRate)
// Sets the clock source and ISR appropriately to do playback, but does not move the RAM pointer.
// Used if we pause playback and want to continue where we left off, or stop overdubbing and jump right back into playback.
{
	audioIsrFunction=AUDIO_ISR_PLAYBACK;	// What should we be doing when we get into the ISR?
	PORTA|=(Om_RAM_OE);						// Tristate the RAM's IO pins.
	PORTA|=(Om_RAM_WE);						// Disbale writes.
	LATCH_DDR=0xFF;							// MCU's data port to outputs.

	if(clockMode==CLK_INTERNAL)			// The internally counted clock is usually associated with MIDI-controlled sampling. 
	{
		TCNT1=0;									// Initialize the counter to 0.
		OCR1A=theRate;				// Set the compare register for the appropriate clock rate.
		TIFR1=0xFF;									// Clear the interrupt flags by writing ones.
		TIMSK1|=(1<<OCIE1A);						// Enable the compare match interrupt.
		TCCR1B=0x09;								// Start the timer in CTC mode, prescaler at 1/1
	}
	else
	{
		TCCR1B|=(1<<ICES1);		// Trigger on a rising edge.
		TIFR1=0xFF;				// Clear interrupts.
		TIMSK1|=(1<<ICIE1);		// Enable Input Capture Interrupt (yo, son, I thought I was the Icy One?)
	}
}

static void StartOverdub(unsigned int theRate)
// Begin recording to ram at the current RAM address from the "overdubbing" analog input.
// Not sure we need to mark the point at which we started overdubbing....
// We make no suppositions about what we were doing before this happened -- it is now the caller's responsibility to stop playing back before they overdub.
{
	overdubStartAddress=ramAddress;			// Begin recording wherever in the sample we happen to be.
	outOfRam=false;							// Plenty of ram left...
	SetAdcChannel(A_OVERDUB);				// Point ADC at the sampler's output.
	audioIsrFunction=AUDIO_ISR_OVERDUB;		// Set the analog clock ISR to do overdubbing.

	PORTA|=(Om_RAM_OE);						// Tristate the RAM's IO pins.
	PORTA|=(Om_RAM_WE);						// Disbale writes.
	LATCH_DDR=0xFF;							// MCU's data port to outputs.

	if(clockMode==CLK_INTERNAL)			// The internally counted clock is usually associated with MIDI-controlled sampling. 
	{
		TCNT1=0;					// Initialize the counter to 0.
		OCR1A=theRate;				// Set the compare register to our presupposed record rate.
		TIFR1=0xFF;					// Clear the interrupt flags by writing ones.
		TIMSK1|=(1<<OCIE1A);		// Enable the compare match interrupt.
		TCCR1B=0x09;				// Start the timer in CTC mode, prescaler at 1/1
	}
	else
	{
		TCCR1B|=(1<<ICES1);		// Trigger on a rising edge.
		TIFR1=0xFF;				// Clear interrupts.
		TIMSK1|=(1<<ICIE1);		// Enable Input Capture Interrupt (yo, son, I thought I was the Icy One?)
	}
}

/*
static void UnInitSampleClock(void)
{
	PRR|=(1<<PRTIM1);	// Turn the TMR1 power off.
}
*/

static void StopAudio(void)
// This generically stops any audio interrupt activity going on, recording, playback or overdubbing. 
{
	TCCR1B=0;							// Stop the timer.
	TIMSK1=0x00;						// Disable all Timer 1 associated interrupts.
	TIFR1=0xFF;							// Clear the interrupt flags by writing ones.
}

static void InitSampleClock(void)
// Get TMR1 ready to generate some equal temperament, or as equal as I can do (for MIDI)
// Or just turn it on so we can use the Input Capture pin to generate interrupts for the external clock (for the loop sampler).
{
	PRR&=~(1<<PRTIM1);	// Turn the TMR1 power on.
	TIMSK1=0x00;		// Disable all Timer 1 associated interrupts.
	OCR1A=65535;		// Set the compare register arbitrarily
	TCCR1A=0;			// Normal Ports.
	TCCR1B=0;			// Stop the timer.
	TCNT1=0;			// Initialize the counter to 0.
	TIFR1=0xFF;			// Clear the interrupt flags by writing ones.
}

//--------------------------------------
//--------------------------------------
// MIDI Functions
//--------------------------------------
//--------------------------------------
// We use Program Changes to do record and change parameters.  We use notes to play back samples at a certain frequency.
// Not sure what to do with Control Changes yet.

#define		MIDI_RECORD_RATE		((OctaveZeroCompareMatches[C_NOTE])>>5)		// About 9.6k

// Program Change messages:
#define		MIDI_START_RECORDING		0
#define		MIDI_STOP_RECORDING			1
#define		MIDI_LOOP_CONTINUOUS		2
#define		MIDI_LOOP_ONCE				3
#define		MIDI_HALF_SPEED				4
#define		MIDI_NORMAL_SPEED			5
#define		MIDI_ONE_BIT_DEPTH			6
#define		MIDI_NORMAL_BIT_DEPTH		7
#define		MIDI_PLAY_BACKWARDS			8
#define		MIDI_PLAY_FORWARDS			9
#define		MIDI_CANCEL_EFFECTS			10


static const unsigned int OctaveZeroCompareMatches[]=
// This table corresponds to a musical octave (the lowest octave we can generate with a 16-bit compare match timer) in 12 tone equal temperament.
{
	65535,		// This is 300 Hz. (Followed by 600, 1.2k, 2.4k, 4.8k, 9.6k, 19.2k shifted over by 1 to 6 resp -- probably can't handle 19.2k, but maybe.  10-bit resolution at >>6)
	61857,
	58385,
	55108,
	52015,
	49096,
	46340,
	43739,
	41284,
	38967,
	36780,
	34716,
};

enum	// Note names for lookups.
{
	C_NOTE=0,
	D_FLAT_NOTE,
	D_NOTE,
	E_FLAT_NOTE,
	E_NOTE,
	F_NOTE,
	G_FLAT_NOTE,
	G_NOTE,
	A_FLAT_NOTE,
	A_NOTE,
	B_FLAT_NOTE,
	B_NOTE,
};

static unsigned int	GetPlaybackRateFromNote(unsigned char theNote)
// Here we take a midi note number and turn it into the timer one compare match interrupt value.
{
	unsigned char
		theIndex,
		theOctave;

	theOctave=(theNote/12);	// Which octave?
	theIndex=(theNote%12);	// Which note inside the octave?

	return((OctaveZeroCompareMatches[theIndex])>>theOctave);		// Return our value using the lookup table.

}

static void DoMidiSampler(void)
// Handle sampling and playing back samples based on the midi messages we get.
// Do all the other fancy effects from here, too.
// Playing a note stops the recording immediately, and starting recording stops a note.  A note turning off doesn't stop a recording.
// RECORD LIGHT = LED_0
// PLAYBACK LIGHT = LED_1
{
	static MIDI_MESSAGE
		currentMidiMessage;

	static unsigned char
		currentNoteOn;

	static bool
		isSamplePlaying,
		isRecording;

	unsigned int
		pitchWheelValue;

	if(subState==SS_0)
	{
//		Uart0ToMidi();					// Make sure the midi baud rate is going.
		midiChannelNumber=0x01;			// @@@ Single midi channel for now.
		StopReadingPot();				// Make sure this silly business is gone (for now)
		loopOnce=true;					// Generally, play midi samples once when the note is turned on.
		halfBitDepth=false;				// And don't do anything else weird.
		halfSpeed=false;
		backwardsPlayback=false;
		isSamplePlaying=false;
		isRecording=false;
		clockMode=CLK_INTERNAL;			// Use internal timing for MIDI.

		KillLeds();					// All leds off, and no blinking.
		subState=SS_1;
	}
	else if(subState==SS_1)
	{
		if(midiMessagesInFifo)
		{
			GetMidiMessageFromFifo(&currentMidiMessage);

			if(currentMidiMessage.messageType==MESSAGE_TYPE_NOTE_OFF)		//  Note off.  Do it.
			{
				if(isSamplePlaying&&(isRecording==false))					// Are we playing a sample to begin with, and are we not recording right now?
				{
					if(currentMidiMessage.dataByteOne==currentNoteOn)		// Sampler is mono.  Only turn off the last note we turned on.
					{
						StopAudio();										// All samples stop the same way.
						isSamplePlaying=false;

						SetLedsOnOff(ledOnOffMask&~(1<<LED_1));		// Playback light off.
					}
				}
			}
			else if(currentMidiMessage.messageType==MESSAGE_TYPE_NOTE_ON)		// Note on.
			{
				if(isSamplePlaying)									// Are we playing a sample already?
				{
					StopAudio();									// Stop it, get ready to play new note.
					isSamplePlaying=false;
					// No need to turn off light here.
				}

				if(isRecording)							// Are we currently recording?
				{
					StopAudio();						// Stop it, get ready to play new note.
					sampleEndAddress=ramAddress;		// Log the location of end of our sample.
					isRecording=false;
					SetLedsOnOff(ledOnOffMask&~(1<<LED_0));		// Record light off.
				}

				currentNoteOn=currentMidiMessage.dataByteOne;			// This is our new note.
				StartPlayback(GetPlaybackRateFromNote(currentNoteOn));	// Do it.
				isSamplePlaying=true;

				SetLedsOnOff(ledOnOffMask|(1<<LED_1));		// Playback light on.
			}

			else if(currentMidiMessage.messageType==MESSAGE_TYPE_PROGRAM_CHANGE)	// We use program to control sampler parameters and start recordings.
			{
				switch(currentMidiMessage.dataByteOne)
				{
					case MIDI_START_RECORDING:						// Can re-start recording arbitrarily.
					StartRecording(MIDI_RECORD_RATE);
					isRecording=true;
					SetLedsOnOff(ledOnOffMask|(1<<LED_0));	// Record light on.
					break;

					case MIDI_STOP_RECORDING:							// Can only stop a recording if a recording is currently happening.
					if(isRecording)
					{
						StopAudio();
						sampleEndAddress=ramAddress;	// Log the location of end of our sample.
						isRecording=false;
						SetLedsOnOff(ledOnOffMask&~(1<<LED_0));	// Record light off.
					}
					break;

					case MIDI_LOOP_CONTINUOUS:						// Keep playing samples over again until note off.
					loopOnce=false;
					SetLedsOnOff(ledOnOffMask|(1<<LED_4));	//  Effect light on.
					break;

					case MIDI_LOOP_ONCE:							// Play sample once after a note on.
					loopOnce=true;
					SetLedsOnOff(ledOnOffMask&~(1<<LED_4));	//  Effect light off.
					break;

					case MIDI_HALF_SPEED:							// Skrew and chop.
					halfSpeed=true;
					SetLedsOnOff(ledOnOffMask|(1<<LED_5));	//  Effect light on.
					break;

					case MIDI_NORMAL_SPEED:							// Un-skrew.
					halfSpeed=false;
					SetLedsOnOff(ledOnOffMask&~(1<<LED_5));	//  Effect light off.
					break;

					case MIDI_ONE_BIT_DEPTH:						// Crusty.
					halfBitDepth=true;
					SetLedsOnOff(ledOnOffMask|(1<<LED_6));	//  Effect light on.
					break;

					case MIDI_NORMAL_BIT_DEPTH:						// Slightly less crusty.
					halfBitDepth=false;
					SetLedsOnOff(ledOnOffMask&~(1<<LED_6));	//  Effect light off.
					break;

					case MIDI_PLAY_BACKWARDS:						// "Paul is Dead"
					backwardsPlayback=true;
					SetLedsOnOff(ledOnOffMask|(1<<LED_7));	//  Effect light on.
					break;

					case MIDI_PLAY_FORWARDS:						// "This Song is Wack"
					backwardsPlayback=false;
					SetLedsOnOff(ledOnOffMask&~(1<<LED_7));	//  Effect light off.
					break;

					case MIDI_CANCEL_EFFECTS:						// Escape from audio mess please.
					loopOnce=true;
					halfBitDepth=false;
					halfSpeed=false;
					backwardsPlayback=false;
					SetLedsOnOff(ledOnOffMask&~((1<<LED_4)|(1<<LED_5)|(1<<LED_6)|(1<<LED_7)));		//  Effect lights off.
					break;

					default:
					break;
				}
			}

			else if(currentMidiMessage.messageType==MESSAGE_TYPE_CONTROL_CHANGE)	// We use Control Changes to ??
			{
				switch(currentMidiMessage.dataByteOne)
				{
					default:
					break;
				}
			}

			else if(currentMidiMessage.messageType==MESSAGE_TYPE_PITCH_WHEEL)
			{
				pitchWheelValue=((currentMidiMessage.dataByteTwo<<7)+currentMidiMessage.dataByteOne);		// Turn the two data bytes into a single 14-bit number (that's how pitch wheel rolls).

				if(pitchWheelValue!=0x2000)		// Pitch wheel being wanked?
				{
					if(pitchWheelValue<0x2000)	// Lower the note by some amount (add value to the OCR1A).	@@@ This wraps around 0 but it sounds cool.
					{
						OCR1A=(GetPlaybackRateFromNote(currentNoteOn)+(0x2000-pitchWheelValue));
					}
					else						// Pitch the note by some amount (add value to the OCR1A).
					{
						OCR1A=(GetPlaybackRateFromNote(currentNoteOn)-(pitchWheelValue-0x2000));
					}
				}
				else
				{
					OCR1A=(GetPlaybackRateFromNote(currentNoteOn));		// Got told to center the note, just do it.
				}
			}
		}
	}

	if(isRecording)							// Stop recording if we're out of RAM.
	{
		if(outOfRam)
		{
			StopAudio();
			sampleEndAddress=ramAddress;	// Log the location of end of our sample.
			isRecording=false;
			SetLedsOnOff(ledOnOffMask&~(1<<LED_0));		// Record light off.
		}
	}
}

//--------------------------------------
//--------------------------------------
// Caveman Interface Functions
//--------------------------------------
//--------------------------------------

enum	// Substates in the loop sampler.
{
	LOOP_ENTERED=0,
	LOOP_USER_WAIT,
	LOOP_RECORDING,
	LOOP_PLAYING_BACK,
	LOOP_PAUSED,
	LOOP_OVERDUBBING,
};

static void AnalogReadingToLeds(void)
// Take the value on our analog input and display it on the LEDs.
// Do this anytime we see the pot change.
{
	unsigned char
		lastPotValue=0,
		temp;

	if(potValue!=lastPotValue)
	{
		currentSelectedEffect=(potValue/32);		// Divide our pot's throw into 8 sections (assumes 8-bit range).
		temp=(ledOnOffMask&0x1F);					// Make a bitmask from the current ledMask and zero the LEDs we're testing.

		if(currentSelectedEffect&(1<<0))
		{
			temp|=(1<<7);	// OR in this bit to the mask to the LEDs.
		}
		if(currentSelectedEffect&(1<<1))
		{
			temp|=(1<<6);	// OR in this bit to the mask to the LEDs.
		}
		if(currentSelectedEffect&(1<<2))
		{
			temp|=(1<<5);	// OR in this bit to the mask to the LEDs.
		}

		SetLedsOnOff(temp);		// Update the leds.
	}

	lastPotValue=potValue;		// Our data is old news.
}

static void SetAudioEffect(unsigned char theEffect)
// Call this to toggle the different effects in the ISR.
{
	if(theEffect==0)
	// Panic!  Bail out of effects.
	{
		loopOnce=false;
		halfBitDepth=false;
		halfSpeed=false;
		backwardsPlayback=false;
	}
	else if(theEffect==1)
	{
		if(halfBitDepth)
		{
			halfBitDepth=false;
		}
		else
		{
			halfBitDepth=true;
		}
	}
	else if(theEffect==2)
	{
		if(halfSpeed)
		{
			halfSpeed=false;
		}
		else
		{
			halfSpeed=true;
		}
	}
	else if(theEffect==3)
	{
		if(backwardsPlayback)
		{
			backwardsPlayback=false;
		}
		else
		{
			backwardsPlayback=true;
		}
	}
	else if(theEffect==4)		// Kludgy!  Look for a better way of playing a sample once.
	{
		if(backwardsPlayback==true)
		{
			ramAddress=sampleEndAddress;
		}
		else
		{
			ramAddress=0;
		}

		loopOnce=true;
	}
}

static void DoSampler(void)
// This state is the font from which all sampler bullshit flows.
// As WTPA stands now, the switches all basically retain the same functions regardless of what the sampler is currently doing.
// Same goes for the LEDs.  So we've gotten rid of a modal system and now just watch for commands via the buttons or midi and update everything accordingly.
// Also, I'm not sure the blinking really helps the user (it used to indicate "ready" to do something) but the sampler is basically either doing something or ready to do it, with the exception
// power up where it has no sample stored yet and is not ready to play.  Perhaps blinking would be useful to indicate "not ready" since this is uncommon.  It'd only ever be useful for the play indicator, though.

// Wed Apr  8 11:42:07 CDT 2009
// This state is based on the idea that playing, recording, and overdubbing are all discrete things and that if you're doing one you can't be doing another.
{
	static bool
		isPlaying,
		isRecording,
		isOverdubbing;

	if(subState==SS_0)
	// Initialize everything.
	{
		midiChannelNumber=0x01;			// @@@ Single midi channel for now.
		ramAddress=0;					// Beginning of RAM.

		StartReadingPot();				// Get Pot info until we're told otherwise.
		loopOnce=false;					// Don't play single shot samples unless we're told to.
		halfBitDepth=false;				// And don't do anything else weird.
		halfSpeed=false;
		backwardsPlayback=false;

		isPlaying=false;				// Not doing anything yet.
		isRecording=false;
		isOverdubbing=false;

		bank0=true;						// Default to bank 0, and both banks are empty.
		bank0Empty=true;
		bank1Empty=true;	

		clockMode=CLK_EXTERNAL;			// Use caveman clock until you see MIDI messages.

		KillLeds();							// All leds off, and no blinking.
		BlinkLeds(Om_LED_PLAY,(SECOND/8));	// Blink the play LED to indicate we have no sample ready to go in our current bank.
		subState=SS_1;	
	}
	else if(subState==SS_1)
	{
		if(keyState&Im_EFFECT)			// If we're holding the effect switch, our other switches call up patches instead of their normal functions.  It's like a shift key.
		{
		
		}
		else
		{
			if(newKeys&Im_REC)			// Record switch pressed.
			{
				if(isRecording)			// Were we recording already?
				{
					StopAudio();					// Stop recording.
					sampleEndAddress=ramAddress;	// Log the location of end of our sample.
					ledOnOffMask&=~Om_LED_REC;		// Turn off the record light.
					isRecording=false;				// Not recording anymore.

					StartPlayback(0);			// Begin playing back the loop we just recorded (ext clock)
					loopOnce=false;
					ledOnOffMask|=Om_LED_PLAY;	// Turn on the play light.
					isPlaying=true;				// We're officially playing back.
				}
				else							// We're not recording right now, so start doing it.
				{
					if(bank0)					// If we're recording into this bank it is no longer empty.
					{
						bank0Empty=false;
					}
					else
					{
						bank1Empty=false;
					}				

					StopBlinking();		// @@@ Since this bank is no longer empty, we won't need to flash the warning light -- if we ever use blinks more often than this one indicator, we'll need to fix this.

					if(isOverdubbing)			// Were we overdubbing?
					{
						StopAudio();
						ledOnOffMask&=~Om_LED_ODUB;		// Turn off the light.
						isOverdubbing=false;			// Not doing this anymore.						
					}
					else if(isPlaying)						// Were we playing?
					{
						StopAudio();
						ledOnOffMask&=~Om_LED_PLAY;		// Turn off the light.
						isPlaying=false;				// Not doing this anymore.											
					}
				
					StartRecording(0);			// Start recording (ext clock)
					ledOnOffMask|=Om_LED_REC;	// Turn on the light.
					isRecording=true;			// Flag for the rest of the program.				
				}
			}

			else if(newKeys&Im_ODUB)			// Overdub switch pressed.  Odub is similar to record except it takes its recording from a different analog input, and it cannot be invoked unless there is already a sample in the bank.
			{
				if(isOverdubbing)				// Were we overdubbing already?
				{
					StopAudio();					// Stop it.
					ledOnOffMask&=~Om_LED_ODUB;		// Turn off the record light.
					isOverdubbing=false;			// Done with this function.

					ContinuePlayback(0);		// Begin playing back the loop we just recorded (ext clock, not from the beginning)
					ledOnOffMask|=Om_LED_PLAY;	// Turn on the play light.
					loopOnce=false;
					isPlaying=true;				// We're officially playing back.
				}
				else							// We're not recording right now, so start doing it.
				{
					if(((bank0==true)&&(bank0Empty==false))||((bank0==false)&&(bank1Empty==false)))		// Because of how OVERDUB thinks about memory we can't do it unless you there's already a sample in the bank.
					{
						if(isRecording)						// Were we doing this?
						{
							StopAudio();					// Stop recording.
							sampleEndAddress=ramAddress;	// Log the location of end of our sample.
							ledOnOffMask&=~Om_LED_REC;	// Turn off the record light.
							isRecording=false;			// Not recording anymore.
						}
						else if(isPlaying)				// Were we playing?
						{
							StopAudio();
							ledOnOffMask&=~Om_LED_PLAY;		// Turn off the light.
							isPlaying=false;				// Not doing this anymore.											
						}
					
						StartOverdub(0);			// Get bizzy (ext clock).
						ledOnOffMask|=Om_LED_ODUB;	// Turn on the light.
						isOverdubbing=true;			// Flag for the rest of the program.				
					}				
				}			
			}
			else if(newKeys&Im_PLAY_PAUSE)		// Play / Pause switch pressed.  If anything is happening this will stop it.  Otherwise, this will start playing back AS A LOOP.  This will not restart a playing sample from the beginning.
			{
				if(isPlaying)					// Playing?
				{
					StopAudio();
					ledOnOffMask&=~Om_LED_PLAY;		// Turn off the light.
					isPlaying=false;				// Not doing this anymore.											
				}
				else if(isRecording)			// Recording?
				{
					StopAudio();				// Stop recording.
					sampleEndAddress=ramAddress;	// Log the location of end of our sample.
					ledOnOffMask&=~Om_LED_REC;	// Turn off the record light.
					isRecording=false;			// Not recording anymore.
				}
				else if(isOverdubbing)			// Dub dub?
				{
					StopAudio();					// Stop it.
					ledOnOffMask&=~Om_LED_ODUB;		// Turn off the record light.
					isOverdubbing=false;			// Done with this function.				
				}
				else							// Not doing *ish.
				{
					if((bank0==true&&bank0Empty==false)||(bank0==false&&bank1Empty==false))		// Have something to play?
					{
						ContinuePlayback(0);		// ext clock @@@ So as of now, this will begin playback at the END of a sample if we've just finished recording.  Ugly.  Find some way to flag the beginning of a sample.
						ledOnOffMask|=Om_LED_PLAY;	// Turn on the play light.
						loopOnce=false;				// This command plays as a loop.
						isPlaying=true;				// We're officially playing back.				
					}
				}			
			}
			else if(newKeys&Im_SINGLE_PLAY)		// Stop whatever we're doing and play the sample from the beginning, one time.
			{
				if(isPlaying)					// Playing?
				{
					StopAudio();
					ledOnOffMask&=~Om_LED_PLAY;		// Turn off the light.
					isPlaying=false;				// Not doing this anymore.											
				}
				else if(isRecording)			// Recording?
				{
					StopAudio();				// Stop recording.
					sampleEndAddress=ramAddress;	// Log the location of end of our sample.
					ledOnOffMask&=~Om_LED_REC;	// Turn off the record light.
					isRecording=false;			// Not recording anymore.
				}
				else if(isOverdubbing)			// Dub dub?
				{
					StopAudio();					// Stop it.
					ledOnOffMask&=~Om_LED_ODUB;		// Turn off the record light.
					isOverdubbing=false;			// Done with this function.				
				}
			
				else if((bank0==true&&bank0Empty==false)||(bank0==false&&bank1Empty==false))		// Have something to play?
				{
					StartPlayback(0);			// Begin playing back the loop we just recorded (ext clock)
					ledOnOffMask|=Om_LED_PLAY;	// Turn on the play light.
					isPlaying=true;				// We're officially playing back.
					loopOnce=true;				// And do it one time, for your mind.
				}
			}
			else if(newKeys&Im_BANK)		// Toggle between our two banks when this key is pressed.
			{
				if(bank0)			// Were in bank 0, switch to 1.  ### handle bank switching while playing or recording
				{
					bank0=false;
					ledOnOffMask|=Om_LED_BANK;	// Turn on the light.
					if(bank1Empty)
					{
						BlinkLeds(Om_LED_PLAY,(SECOND/8));	// Blink the play LED to indicate we have no sample ready to go in our current bank.
					}
				}				
				else			// Were in bank 1, switch to 0.
				{
					bank0=true;
					ledOnOffMask&=~Om_LED_BANK;	// Turn off the light.
					if(bank0Empty)
					{
						BlinkLeds(Om_LED_PLAY,(SECOND/8));	// Blink the play LED to indicate we have no sample ready to go in our current bank.
					}
				}				
			
			}		
		}
	}

	if(isRecording||isOverdubbing)			// Stop reading the pot if we're using the adc for something else and stop recording if we're out of RAM.
	{
		StopReadingPot();					// This will stop the updates to the pot if we're recording or overdubbing.  Since we've already changed ADC channels we're OK waiting until this point in the state machine to give this command.

		if(outOfRam)
		{
			StopAudio();
			if(isRecording)
			{
				sampleEndAddress=ramAddress;	// Log the location of end of our sample.
				isRecording=false;
				ledOnOffMask&=~Om_LED_REC;	// Turn off the record light.
			}
			else
			{
				ledOnOffMask&=~Om_LED_ODUB;		// Turn off the record light.
				isOverdubbing=false;			// Done with this function.							
			}

			ledOnOffMask|=Om_LED_OUT_OF_MEM;	// Turn on the light.
		}
	}
	else
	{
		if(updatePot==false)
		{
			StartReadingPot();	// If we aren't doing anything else important with the ADC start reading the pot again.
		}
	}	

	AnalogReadingToLeds();				// Display our pot's relative value on the leds.
}

//--------------------------------------
//--------------------------------------
// DAC Testing.
//--------------------------------------
//--------------------------------------

static void DoSawtooth(void)
// See if we can get some audio out.  And look good doing it.
{
	if(subState==SS_0)
	{
		KillLeds();										// Start with LEDs off.
		BlinkLeds(((1<<LED_6)|(1<<LED_7)),(SECOND/8));	// Blink Leds 6, 7.
		subState=SS_1;
	}
	else if(subState==SS_1)
	{
		audioIsrFunction=AUDIO_ISR_SAWTOOTH;	// Set the external analog clock interrupt vector to make sawtooth waves.
		TCCR1B|=(1<<ICES1);						// Trigger on a rising edge.
		TIFR1=0xFF;								// Clear interrupts.
		TIMSK1|=(1<<ICIE1);						// Enable Input Capture Interrupt (yo, son, I thought I was the Icy One?)

		subState=SS_2;					// And wait forever.
	}
	else if(subState==SS_2)
	{
		if(newKeys&Im_SWITCH_0)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_0));	// Toggle the LED.
		}
		if(newKeys&Im_SWITCH_1)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_1));	// Toggle the LED.
		}
		if(newKeys&Im_SWITCH_2)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_2));	// Toggle the LED.
		}
		if(newKeys&Im_SWITCH_3)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_3));	// Toggle the LED.
		}
		if(newKeys&Im_SWITCH_4)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_4));	// Toggle the LED.
		}
		if(newKeys&Im_SWITCH_5)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_5));	// Toggle the LED.
		}
	}
}

static void MidiOutputTestBinnis(void)
{
	if(subState==SS_0)
	{
		midiChannelNumber=0x01;			// @@@ Single midi channel for now.
		StopReadingPot();				// Make sure this silly business is gone (for now)
		KillLeds();						// All leds off, and no blinking.
		subState=SS_1;
	}
	else
	{
		if(newKeys&Im_SWITCH_0)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_0));	// Toggle the LED.
			// Hardcoded Note On, middle C, Velocity 64, Midi channel 1.
			Uart0SendByte(0x91);		// @@@
			Uart0SendByte(60);			// @@@
			Uart0SendByte(64);			// @@@
			
		}
		if(newKeys&Im_SWITCH_1)
		{
			SetLedsOnOff(ledOnOffMask^(1<<LED_1));	// Toggle the LED.
			// Hardcoded Note Off, middle C, Velocity 0, Midi channel 1.
			Uart0SendByte(0x91);		// @@@
			Uart0SendByte(60);			// @@@
			Uart0SendByte(0);			// @@@
		}
	}
}


static void DoTurningOn(void)
// Make all our initial state decisions.
// Give switches time to settle.
{
	if(subState==SS_0)
	{
		SetTimer(TIMER_1,(SECOND/8));
		subState=SS_1;
	}
	else
	{
		if(CheckTimer(TIMER_1))
		{
			if(keyState&Im_SWITCH_0)
			{
				SetState(DoSawtooth);
			}
			else if(keyState&Im_SWITCH_1)
			{
				SetState(DoMidiSampler);
			}
			else
			{
				SetState(DoSampler);			
			}
			
			printf("Can I bring my gat?\n");
		
		}
	}

//		SetState(DoSawtooth);
//		SetState(DoLoopSampler);
//		printf("Yo Joe.\n");
//		SetState(DoMidiSampler);
//		SetState(MidiOutputTestBinnis);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Local Software Clock stuff.
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

static void HandleSoftclock(void)
{
	if(TIFR0&(1<<TOV0))		// Got a timer overflow flag?
	{
		TIFR0 |= (1<<TOV0);		// Reset the flag (by writing a one).
		systemTicks++;			// Increment the system ticks.
	}
}

static void InitSoftclock(void)
// Wed Dec  3 22:28:06 CST 2008
// I've changed the way the softclock works from the last rev.  It's no longer and interrupt based thing, so it doesn't potentially get in the way of the audio interrupts.
// Also, since hardware TIMR1 is needed for bigger and better things than keeping human-time, we're going to set the softclock timer up to poll a flag.
// This means we don't steal cycles from any other ISRs, but it also means that if we write dumb code (if we hang the main loop for more than a tick time)
// we might miss a systemTick.
// NOTE:  w/ TMR0 running at 1/256 prescale at 20MHz, our smallest time unit is 3.2768mSecs.
// NOTE:  w/ 16-bit system ticks we can only time 214 seconds at this rate.
{
	PRR&=~(1<<PRTIM0);	// Turn the TMR0 power on.
	TIMSK0=0x00;		// Disable all Timer 0 associated interrupts.
	TCCR0A=0;			// Normal Ports.
	TCNT0=0;			// Initialize the counter to 0.
	TIFR0=0xFF;			// Clear the interrupt flags by writing ones.
	systemTicks=0;
	TCCR0B=0x04;		// Start the timer in Normal mode, prescaler at 1/256
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Program main loop.
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

int main (void)
// Initialize this mess.
{
	PRR=0xFF;			// Power off everything, let the initialization routines turn on modules you need.
							
	// Set the DDRs for all RAM, DAC, latch related pins here.  Switches and ADC and everything else with a specific init routine will be intialized separately.

	PORTD=0x9F;			// Turn on pullups for everything except ACLK and the UART inits will get rid of the ones we don't need, and leave PD5 low.
	DDRD=0x20;			// PORTD is mostly inputs with the exception of the UARTs and the LED Latch enable on PD5.

	PORTA=0x06;			// OE and WE high, latch enables low, everything else don't care.
	DDRA=0x3E;			// PORTA is digital outputs (latch strobe lines and OE/WE lines) and analog ins (on 0, 6, 7)

	DDRB=0xFF;			// Latch port to OP.
	LATCH_PORT=0;		// And low (PORTB).

	DDRC=0x07;			// PORTC is switch inputs (initialized later) and direct address line outputs which must be initialized here.
	PORTC=0x00;			// 0, 1, 2 are the address lines.

	InitSwitches();
	InitLeds();
	InitMidi();					// Get the MIDI stack initialized.
	InitUart0();
	InitUart1();
	InitAdc();
	InitSoftclock();
//	InitSerialTransmitFifo();	@@@
	InitSampleClock();	// Turns on TIMER1 and gets it ready to generate interrupts.

	ramAddress=0;
	dacByte=0;
	newKeys=0;
	keyState=0;

	sei();						// THE ONLY PLACE we should globally enable interrupts in this code.

 	SetState(DoTurningOn);

	while(1)
	{
		HandleSwitches();		// Flag newKeys.
		HandleSoftclock();		// Keep the timer timing.
		HandleLeds();			// Keep LEDs updated.
		HandleReadingPot();		// Read the analog pot input if appropros.  Oui oui!

		if(Uart0GotByte())		// Handle receiving midi messages: Parse any bytes coming in over the UART into MIDI messages we can use.
		{
			HandleMidiByte(Uart0GetByte());		// Deal with the UART message and put it in the incoming MIDI FIFO if relevant.
		}

//		if(serialBytesInFifo>0)		// Got something to transmit to the Helper?	@@@
//		{
//			if(UCSR1A&(1<<UDRE1))	// UART transmitter ready for new data?
//			{
//				UDR1=GetByteFromSerialTransmitFifo();	// Send the byte.
//			}
//		}

		State();				// Execute the current program state.
	}
	return(0);
}

